Package: segmented
Type: Package
Title: Regression Models with Break-Points / Change-Points Estimation
        (with Possibly Random Effects)
Version: 2.2-1
Date: 2026-01-29
Authors@R: c(person(given = c("Vito","M.","R."), family = "Muggeo", role =
        c("aut", "cre"), email = "vito.muggeo@unipa.it", comment=c(ORCID="0000-0002-3386-4054")))
Maintainer: Vito M. R. Muggeo <vito.muggeo@unipa.it>
Description: Fitting regression models where, in addition to possible linear terms, one or more covariates have segmented (i.e., broken-line or piece-wise linear) or stepmented (i.e. piece-wise constant) effects. Multiple breakpoints for the same variable are allowed. 
  The estimation method is discussed in Muggeo (2003, <doi:10.1002/sim.1545>) and 
  illustrated in Muggeo (2008, <https://www.r-project.org/doc/Rnews/Rnews_2008-1.pdf>). An approach for hypothesis testing is presented 
  in Muggeo (2016, <doi:10.1080/00949655.2016.1149855>), and interval estimation for the breakpoint is discussed in Muggeo (2017, <doi:10.1111/anzs.12200>). 
  Segmented mixed models, i.e. random effects in the change point, are discussed in Muggeo (2014, <doi:10.1177/1471082X13504721>).
  Estimation of piecewise-constant relationships and changepoints (mean-shift models) is 
  discussed in Fasola et al. (2018, <doi:10.1007/s00180-017-0740-4>).
Depends: R (>= 3.5.0), MASS, nlme
License: GPL
NeedsCompilation: no
Packaged: 2026-01-29 14:50:37 UTC; vito
Author: Vito M. R. Muggeo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3386-4054>)
RoxygenNote: 7.3.1
Repository: CRAN
Date/Publication: 2026-01-29 15:40:03 UTC
Built: R 4.6.0; ; 2026-04-25 23:17:45 UTC; unix
